/* =========================================================================
 *
 *             Nanite Systems Applied Research Execution System
 *  
 *            Copyright (c) 2022–2023 Nanite Systems Corporation
 *  
 * =========================================================================
 *
 *  Navigation Event Handlers
 *
 *  This program is covered under the terms of the ARES Software Copyright
 *  License, Section 4 (ASCL-iv). It is a closed-source, proprietary,
 *  commercial product and may not be distributed.
 *  
 *  To see the full text of the ASCL, type 'help license' on any standard
 *  ARES distribution, or visit http://nanite-systems.com/ASCL for the
 *  current version.
 *
 *  DISCLAIMER
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS
 *  IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 *  TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 *  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 *  DAMAGES HOWEVER CAUSED ON ANY THEORY OF LIABILITY ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGE.
 *
 * =========================================================================
 *
 */
 
 /*
	Reader, do not implement EXT_EVENT_HANDLER in your application unless it is absolutely, strictly necessary. Use kernel timers if the interval is above ten seconds, and cyanide tablets otherwise. Applications should not be performing any task that requires a high-speed timer.
	
	Anyway, here's a high-speed timer in an application for a task that definitely requires it.
 */

	timer() {
		vector delta;
	
		if(next_nav_dest != ZV) {
			vector mybb = llGetAgentSize(avatar);
			vector adjusted_next_nav_dest = next_nav_dest + <0, 0, mybb.z * 0.5>;
			delta = adjusted_next_nav_dest - llGetPos();
			float dist = llVecMag(delta);
			
			float deviation;
			if(follow_target) {
				deviation = llVecDist(object_pos(follow_target), nav_path_dest);
			}
			
			if(deviation > 1) {
				// echo("target moved; need to recalculate");
				// llStopMoveToTarget();
				nav_path = [];
				next_nav_dest = ZV;
				moving = 2;
			} else if(dist < 1 || (dist < leash_length && llVecDist(nav_path_dest, next_nav_dest) < 1)) {
				// echo("selecting next node");
				moving = 0;
				llStopMoveToTarget();
				next_nav_dest = getv(nav_path, 0);
				nav_path = delitem(nav_path, 0);
				/* if(next_nav_dest == ZV) {
					echo("[nav] Nav finished: target reached.");
					stop_nav();
					return;
				} */
			} else {
				// echo("seeking by path");
				llMoveToTarget(adjusted_next_nav_dest, dist / speed);
				moving = 1;
			}
		}
		
		if(follow_target != "" && follow_target != NULL_KEY && next_nav_dest == ZV) {
			vector target_pos = object_pos(follow_target);
			if(target_pos == ZV) {
				echo("[nav] Follow aborted: target gone.");
				stop_follow();
				return;
			} else {
				vector target_bb = llGetAgentSize(follow_target);
				float target_vertical_adjust = target_bb.z * 0.5;
				float my_vertical_adjust;
				
				vector my_pos = llGetPos();
				
				if(target_vertical_adjust) { // following an avatar
					vector my_bb = llGetAgentSize(avatar);
					my_vertical_adjust = my_bb.z * 0.5;
					
					target_pos.z -= target_vertical_adjust;
					my_pos.z -= my_vertical_adjust;
				}
				
				delta = target_pos - my_pos;
				
				float dist = llVecMag(delta);
			
				if(dist > leash_length) {
					nav_path = delitem(construct_path(follow_target), LAST);
					if(count(nav_path)) {
						nav_path_dest = target_pos;
						next_nav_dest = getv(nav_path, 0);
						nav_path = delitem(nav_path, 0);
						delta = next_nav_dest - llGetPos();
						// echo("selected node");
					} else {
						float go_dist = (dist - leash_length * 0.5);
						llMoveToTarget(my_pos + <0, 0, my_vertical_adjust> + llVecNorm(delta) * go_dist, go_dist / speed);
						moving = 1;
						// echo("seeking directly");
					}
				} else if(moving) {
					llStopMoveToTarget();
					moving = 2;
					// echo("arrived");
				}
			}
		}
		
		if(moving) {
			vector my_dir = llRot2Fwd(getr(geto(avatar, [OBJECT_ROT]), 0));
			float my_facing = llAtan2(my_dir.x, my_dir.y);
			float correct_facing = llAtan2(delta.x, delta.y);
			float facing_delta = llFabs(my_facing - correct_facing);
			if(facing_delta > 0.1 && facing_delta < 3.04)
				echo("@setrot:" + (string)correct_facing + "=force");
		
			if(moving == 2)
				moving = 0;
		}
	}
	