
#ifndef UTILS
#define UTILS

/*

   ***********************************************************
   *                                                         *
   *    NANITE SYSTEMS ADVANCED TACTICAL OPERATING SYSTEM    *
   *                                                         *
   *                     UTILS COMPONENT                     *
   *                                                         *
   *  Copyright, (C) Nanite Systems Corp., 1984-85, 2017-22  *
   *                                                         *
   *     Copyright, (C) University of Michigan 1977-1981     *
   *                                                         *
   ***********************************************************
   
   THE SOURCE CODE FOR THIS LIBRARY COMPONENT IS PROVIDED TO
   NANITE SYSTEMS CUSTOMERS FOR DEVELOPMENT OF COMPATIBLE
   SOFTWARE.
   
   BY USING THIS CODE, YOU AGREE NOT TO MODIFY OR DISTRIBUTE
   IT FOR COMMERCIAL GAIN, EXCEPT IN A COMPILED PRODUCT. YOU
   MAY PROVIDE THE SOURCE CODE OF THIS FILE FOR FREE OR
   INCLUDED WITH YOUR OWN DEVELOPMENT PACKAGE AS LONG AS THIS
   NOTICE IS INCLUDED AND THE REST OF THE FILE IS UNMODIFIED.
   
   INTERACTIONS WITH OTHER COMPONENTS OF THE OPERATING SYSTEM
   ARE SUBJECT TO THE APPROPRIATE END-USER LICENSE AGREEMENT
   (EULA).
   
   IDV:				F0960303D0002
   VENDOR DUNS:     005128988
   
*/

#define contains(lis, item) ~llListFindList(lis, (list)(item))
#define index(lis, item) llListFindList(lis, (list)(item))

#define strlen llStringLength
#define substr(iijstr, iij, iik) llGetSubString(iijstr, iij, iik)
#define strpos(iijstr, needleiij) llSubStringIndex(iijstr, needleiij)
#define strdelete llDeleteSubString
#define delstring llDeleteSubString

#define ucount(lis) (lis != [])
#define count llGetListLength
#define geti llList2Integer
#define gets llList2String
#define getk llList2Key
#define getv llList2Vector
#define getr llList2Rot
#define getf llList2Float
// #define sublist(lis, aij, bij) llList2List(lis, aij, bij)
#define sublist llList2List

#define alter llListReplaceList
#define getjs llJsonGetValue
#define setjs llJsonSetValue
#define etype llGetListEntryType
#define jstype llJsonValueType
#define echo llOwnerSay
#define tell llRegionSayTo

#define list2js llList2Json
#define js2list llJson2List
#define jsarray(...) llList2Json(JSON_ARRAY, __VA_ARGS__)
#define jsobject(...) llList2Json(JSON_OBJECT, __VA_ARGS__)
#define jskeys(__jso) llList2ListStrided(llJson2List(__jso), 0, LAST, 2)

#define linked llMessageLinked

#define split(lisstr, sepij) llParseString2List(lisstr, (list)(sepij), [])
#define splitnulls(lisstr, sepij) llParseStringKeepNulls(lisstr, (list)(sepij), [])
#define concat llDumpList2String
#define delrange llDeleteSubList
#define delitem(lis, iij) llDeleteSubList(lis, iij, iij)
#define insert llListInsertList
#define shuffle llListRandomize

#define replace(haystack, old, new) llReplaceSubString(haystack, old, new, 0)
// #define replace(haystack, old, new) concat(splitnulls(haystack, old), new)

// convert to and from NS-style space-separated string vectors:
#define vec2str(iijvec) ((string)iijvec.x + " " + (string)iijvec.y + " " + (string)iijvec.z)
#define str2vec(__str) (vector)("<" + replace(__str, " ", ",") + ">")
// "Nothing is true - all is permissible."
//     - Hassan i Sabbah

#define format_percentage(xxx) ((string)((integer)((xxx) * 100)) + "%")

// format_float(0.010999, 3) == "0.010"
#define format_float(xxx, yyy) llGetSubString( (string) xxx, 0, llSubStringIndex( (string) xxx, ".") + yyy )

list list_union(list ly, list lx) { // add the lists, eliminating duplicates
    list lz = [];
	integer x = count(ly);
	
    while(x--)
        if(!~llListFindList(lx, sublist(ly, x, x)))
            lz = sublist(ly, x, x) + lz;
	
    return lx + lz;
}

list list_exclude(list ly, list lx) { // remove entries in second list from first
	list lz = [];
	integer x = count(ly);
	
    while(x--)
        if(!~llListFindList(lx, sublist(ly, x, x)))
            lz = sublist(ly, x, x) + lz;
	
    return lz;
}

list list_intersect(list ly, list lx) { // remove entries not in both lists
	list lz = [];
	integer x = count(ly);
	
    while(x--)
        if(~llListFindList(lx, sublist(ly, x, x)))
            lz = sublist(ly, x, x) + lz;
	
    return lz;
}

integer validate_key(key k) { // slow, but correct
	string sk = llToLower((string)k);
	return (
		strlen(sk) == 36
		&& !count(split((string)llParseString2List((string)llParseString2List(sk, ["0","1","2","3","4","5","6","7"], []), ["8", "9", "a", "b", "c", "d", "e", "f"], []), "-")) // llParseString2List can't handle more than 8 separators
		&& count(splitnulls(sk, "-")) == 5
		&& (substr(sk, 8, 8) + substr(sk, 13, 13) + substr(sk, 18, 18) + substr(sk, 23, 23)) == "----"
	);
}

// iterates over 'input', translating 'keys' into 'values' by matching indices
// e.g. remap([0, 1, 2], ["a", "b", "c"], [2, 2, 1]) ==> ["c", "c", "b"]
list remap(list keys, list values, list input) {
	list results;
	integer i = count(input);
	while(i--) {
		integer j = llListFindList(keys, sublist(input, i, i));
		if(~j)
			results = sublist(values, j, j) + results;
	}
	
	return results;
}

// unicode management functions:
string strleft_byte(string str, integer bytes) {
    string temp = llStringToBase64(str);
    if(strlen(temp) <= (bytes * 8) / 6)
        return str;
    temp = llBase64ToString(substr(temp+"==", (bytes * -2) % 3, (bytes * 4 - 1) / 3));
    integer i = strlen(temp) - 1;
    if(substr(temp, i, i) != substr(str, i, i))
        return llDeleteSubString(temp, i, i);//last char was multi-byte and was truncated.
    return temp;
}

integer strlen_byte(string str) {
	return (( (3 * strpos(llStringToBase64(str)+"=", str = "=")) >> 2 ));
}

string format_time(float number) {
	integer negative;
	if(number < 0) {
		negative = TRUE;
		number = llFabs(number);
	}
    string output;

	number += 0.5;
	
    integer secs = (integer)(number) % 60;
    integer minutes = (integer)(number / 60) % 60;
    integer hours = (integer)(number / 3600) % 24;
    integer days = (integer)(number / 86400);
    
    if(days == 1)
		output = "1 day, ";
    else if(days)
		output = (string)days + " days, ";
    
    // if(hours > 0 || days > 0)
	output += (string)hours + ":";
    
    if(minutes < 10)
		output += "0";
	
    output += (string)minutes + ":";
    
    if(secs < 10)
		output += "0";
		
    output += (string)secs;
	
	if(negative)
		output = "-(" + output + ")";
	
    return output;
}

// from http://wiki.secondlife.com/wiki/Efficient_Hex:

string hex(integer bits) {
    string nybbles = "";
    do {
        integer lsn = bits & 0xF; // least significant nybble
        nybbles = substr("0123456789abcdef", lsn, lsn) + nybbles;
    } while (bits = (0xfffFFFF & (bits >> 4)));
    return nybbles;
}

// various semantic negative one replacements:
#define NOWHERE 0xFFFFFFFF
#define LAST 0xFFFFFFFF

// you better believe this is necessary:
#define SAFE_EOF llChar(4)

#endif // UTILS
